@@ -195,6 +195,7 @@ static const struct spa_dict_item node_info_items[] = {
 	{ SPA_KEY_DEVICE_API, "alsa" },
 	{ SPA_KEY_MEDIA_CLASS, "Midi/Bridge" },
 	{ SPA_KEY_NODE_DRIVER, "true" },
+	{ "priority.driver", "1" },
 };
 
 static void emit_node_info(struct seq_state *this, bool full)
@@ -676,7 +677,7 @@ impl_node_port_set_param(void *object,
 {
 	struct seq_state *this = object;
 	struct seq_port *port;
-	int res;
+	int res = 0;
 
 	spa_return_val_if_fail(this != NULL, -EINVAL);
 
