@@ -92,6 +92,7 @@ struct impl {
 	struct spa_source source;
 	struct spa_source notify;
 	unsigned int use_acp:1;
+	unsigned int expose_busy:1;
 };
 
 static int impl_udev_open(struct impl *this)
@@ -329,6 +330,27 @@ static int get_num_compress_offload_devices(unsigned int card_nr)
 	return errno != 0 ? -errno : num_dev;
 }
 
+static int check_udev_environment(struct udev *udev, const char *devname)
+{
+	char path[PATH_MAX];
+	struct udev_device *dev;
+	int ret = 0;
+
+	/* Check for ACP_IGNORE on a specific PCM device (not the whole card) */
+	spa_scnprintf(path, sizeof(path), "/sys/class/sound/%s", devname);
+
+	dev = udev_device_new_from_syspath(udev, path);
+	if (dev == NULL)
+		return 0;
+
+	if (udev_device_get_property_value(dev, "ACP_IGNORE"))
+		ret = -ENXIO;
+
+	udev_device_unref(dev);
+
+	return ret;
+}
+
 static int check_pcm_device_availability(struct impl *this, struct card *card,
                                          int *num_pcm_devices)
 {
@@ -360,6 +382,8 @@ static int check_pcm_device_availability(struct impl *this, struct card *card,
 	 */
 
 	res = 0;
+	if (this->expose_busy)
+		return res;
 
 	spa_scnprintf(path, sizeof(path), "/proc/asound/card%u", card->card_nr);
 
@@ -376,6 +400,9 @@ static int check_pcm_device_availability(struct impl *this, struct card *card,
 				card->card_nr, entry->d_name+3);
 		if (check_device_pcm_class(path) < 0)
 			continue;
+		/* Check udev environment */
+		if (check_udev_environment(this->udev, path) < 0)
+			continue;
 
 		/* Check busy status */
 		spa_scnprintf(path, sizeof(path), "/proc/asound/card%u/%s",
@@ -1117,6 +1144,8 @@ impl_init(const struct spa_handle_factory *factory,
 	if (info) {
 		if ((str = spa_dict_lookup(info, "alsa.use-acp")) != NULL)
 			this->use_acp = spa_atob(str);
+		else if ((str = spa_dict_lookup(info, "alsa.udev.expose-busy")) != NULL)
+			this->expose_busy = spa_atob(str);
 	}
 
 	return 0;
