@@ -1,26 +1,6 @@
-/* GStreamer
- *
- * Copyright © 2018 Wim Taymans
- *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice (including the next
- * paragraph) shall be included in all copies or substantial portions of the
- * Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
- */
+/* GStreamer */
+/* SPDX-FileCopyrightText: Copyright © 2018 Wim Taymans */
+/* SPDX-License-Identifier: MIT */
 
 #include <stdio.h>
 
@@ -749,7 +729,7 @@ handle_id_prop (const struct spa_pod_prop *prop, const char *key, id_to_string_f
   uint32_t i, n_items, choice;
 
   val = spa_pod_get_values(&prop->value, &n_items, &choice);
-  if (val->type != SPA_TYPE_Id)
+  if (val->type != SPA_TYPE_Id || n_items == 0)
           return;
 
   id = SPA_POD_BODY(val);
@@ -790,7 +770,7 @@ handle_int_prop (const struct spa_pod_prop *prop, const char *key, GstCaps *res)
   uint32_t i, n_items, choice;
 
   val = spa_pod_get_values(&prop->value, &n_items, &choice);
-  if (val->type != SPA_TYPE_Int)
+  if (val->type != SPA_TYPE_Int || n_items == 0)
           return;
 
   ints = SPA_POD_BODY(val);
@@ -834,7 +814,7 @@ handle_rect_prop (const struct spa_pod_prop *prop, const char *width, const char
   uint32_t i, n_items, choice;
 
   val = spa_pod_get_values(&prop->value, &n_items, &choice);
-  if (val->type != SPA_TYPE_Rectangle)
+  if (val->type != SPA_TYPE_Rectangle || n_items == 0)
           return;
 
   rect = SPA_POD_BODY(val);
@@ -887,7 +867,7 @@ handle_fraction_prop (const struct spa_pod_prop *prop, const char *key, GstCaps
   uint32_t i, n_items, choice;
 
   val = spa_pod_get_values(&prop->value, &n_items, &choice);
-  if (val->type != SPA_TYPE_Fraction)
+  if (val->type != SPA_TYPE_Fraction || n_items == 0)
           return;
 
   fract = SPA_POD_BODY(val);
