@@ -29,7 +29,7 @@
 
 #include "flatpak-utils.h"
 
-/** \page page_module_access PipeWire Module: Access
+/** \page page_module_access Access
  *
  *
  * The `access` module performs access checks on clients. The access check
@@ -50,6 +50,7 @@
  * - If `access.legacy` is enabled, the value is:
  *
  *     - `"flatpak"`: if client is a Flatpak client
+ *     - `"unrestricted"`: if \ref PW_KEY_CLIENT_ACCESS client property is set to `"allowed"`
  *     - Value of \ref PW_KEY_CLIENT_ACCESS client property, if set
  *     - `"unrestricted"`: otherwise
  *
@@ -63,6 +64,10 @@
  * `pipewire.access.portal.app_id` property is to the Flatpak application ID, if
  * found. In addition, `pipewire.sec.flatpak` is set to `true`.
  *
+ * ## Module Name
+ *
+ * `libpipewire-module-access`
+ *
  * ## Module Options
  *
  * Options specific to the behavior of this module
@@ -195,12 +200,16 @@ context_check_access(void *data, struct pw_impl_client *client)
 		else
 			access = ACCESS_DEFAULT;
 	} else {
-		if (sandbox_flatpak)
+		if (sandbox_flatpak) {
 			access = ACCESS_FLATPAK;
-		else if ((str = pw_properties_get(props, PW_KEY_CLIENT_ACCESS)) != NULL)
-			access = str;
-		else
+		} else if ((str = pw_properties_get(props, PW_KEY_CLIENT_ACCESS)) != NULL) {
+			if (spa_streq(str, "allowed"))
+				access = ACCESS_UNRESTRICTED;
+			else
+				access = str;
+		} else {
 			access = ACCESS_UNRESTRICTED;
+		}
 	}
 
 	/* Handle resolution */
