@@ -100,6 +100,7 @@ struct stream {
 	struct spa_io_buffers *io;
 	struct spa_io_rate_match *rate_match;
 	uint32_t rate_queued;
+	uint64_t rate_size;
 	struct {
 		struct spa_io_position *position;
 	} rt;
@@ -420,9 +421,8 @@ static struct buffer *get_buffer(struct pw_stream *stream, uint32_t id)
 
 static inline uint32_t update_requested(struct stream *impl)
 {
-	uint32_t index, id, res = 0;
+	uint32_t index, id;
 	struct buffer *buffer;
-	struct spa_io_rate_match *r = impl->rate_match;
 
 	if (spa_ringbuffer_get_read_index(&impl->dequeued.ring, &index) < 1) {
 		pw_log_debug("%p: no free buffers %d", impl, impl->n_buffers);
@@ -431,15 +431,11 @@ static inline uint32_t update_requested(struct stream *impl)
 
 	id = impl->dequeued.ids[index & MASK_BUFFERS];
 	buffer = &impl->buffers[id];
-	if (r) {
-		buffer->this.requested = r->size;
-		res = r->size > 0 ? 1 : 0;
-	} else {
-		buffer->this.requested = impl->quantum;
-		res = 1;
-	}
+	buffer->this.requested = impl->rate_size;
+
 	pw_log_trace_fp("%p: update buffer:%u req:%"PRIu64, impl, id, buffer->this.requested);
-	return res;
+
+	return buffer->this.requested > 0 ? 1 : 0;
 }
 
 static int
@@ -683,8 +679,13 @@ static inline void copy_position(struct stream *impl, int64_t queued)
 		impl->time.queued = queued;
 		impl->quantum = p->clock.duration;
 	}
-	if (SPA_LIKELY(impl->rate_match != NULL))
+	if (SPA_LIKELY(impl->rate_match != NULL)) {
 		impl->rate_queued = impl->rate_match->delay;
+		impl->rate_size = impl->rate_match->size;
+	} else {
+		impl->rate_queued = 0;
+		impl->rate_size = impl->quantum;
+	}
 	SPA_SEQ_WRITE(impl->seq);
 }
 
@@ -717,11 +718,11 @@ static int impl_send_command(void *object, const struct spa_command *command)
 				if (impl->io != NULL)
 					impl->io->status = SPA_STATUS_NEED_DATA;
 			}
-			else if (!impl->process_rt && !impl->driving) {
+			else {
 				copy_position(impl, impl->queued.incount);
-				call_process(impl);
+				if (!impl->process_rt && !impl->driving)
+					call_process(impl);
 			}
-
 			stream_set_state(stream, PW_STREAM_STATE_STREAMING, 0, NULL);
 		}
 		break;
@@ -856,9 +857,12 @@ static void clear_buffers(struct pw_stream *stream)
 		if (SPA_FLAG_IS_SET(b->flags, BUFFER_FLAG_MAPPED)) {
 			for (j = 0; j < b->this.buffer->n_datas; j++) {
 				struct spa_data *d = &b->this.buffer->datas[j];
-				pw_log_debug("%p: clear buffer %d mem",
-						stream, b->id);
-				unmap_data(impl, d);
+				if (SPA_FLAG_IS_SET(d->flags, SPA_DATA_FLAG_MAPPABLE) ||
+				    (mappable_dataTypes & (1<<d->type)) > 0) {
+					pw_log_debug("%p: clear buffer %d mem",
+							stream, b->id);
+					unmap_data(impl, d);
+				}
 			}
 		}
 	}
@@ -1035,7 +1039,7 @@ static int impl_node_process_input(void *object)
 	struct stream *impl = object;
 	struct pw_stream *stream = &impl->this;
 	struct spa_io_buffers *io = impl->io;
-	struct buffer *b;
+	struct buffer *b = NULL;
 
 	if (io == NULL)
 		return -EIO;
@@ -1052,10 +1056,13 @@ static int impl_node_process_input(void *object)
 				SPA_ATOMIC_INC(b->busy->count);
 		}
 	}
-	if (!queue_is_empty(impl, &impl->dequeued)) {
-		copy_position(impl, impl->dequeued.incount);
+
+	copy_position(impl, impl->dequeued.incount);
+	if (b != NULL)
+		b->this.time = impl->time.now;
+
+	if (!queue_is_empty(impl, &impl->dequeued))
 		call_process(impl);
-	}
 
 	if (io->status != SPA_STATUS_NEED_DATA || io->buffer_id == SPA_ID_INVALID) {
 		/* pop buffer to recycle */
@@ -1264,6 +1271,10 @@ static int node_event_param(void *object, int seq,
 		}
 
 		pod = spa_pod_get_values(type, &n_vals, &choice);
+		if (n_vals == 0) {
+			free(c);
+			return -EINVAL;
+		}
 
 		c->type = SPA_POD_TYPE(pod);
 		if (spa_pod_is_float(pod))
@@ -1523,10 +1534,12 @@ stream_new(struct pw_context *context, const char *name,
 
 	if (pw_properties_get(props, PW_KEY_STREAM_IS_LIVE) == NULL)
 		pw_properties_set(props, PW_KEY_STREAM_IS_LIVE, "true");
-	if (pw_properties_get(props, PW_KEY_NODE_NAME) == NULL && extra) {
-		str = pw_properties_get(extra, PW_KEY_APP_NAME);
-		if (str == NULL)
-			str = pw_properties_get(extra, PW_KEY_APP_PROCESS_BINARY);
+	if ((str = pw_properties_get(props, PW_KEY_NODE_NAME)) == NULL) {
+		if (extra) {
+			str = pw_properties_get(extra, PW_KEY_APP_NAME);
+			if (str == NULL)
+				str = pw_properties_get(extra, PW_KEY_APP_PROCESS_BINARY);
+		}
 		if (str == NULL)
 			str = name;
 		pw_properties_set(props, PW_KEY_NODE_NAME, str);
@@ -2347,13 +2360,14 @@ int pw_stream_get_time_n(struct pw_stream *stream, struct pw_time *time, size_t
 {
 	struct stream *impl = SPA_CONTAINER_OF(stream, struct stream, this);
 	uintptr_t seq1, seq2;
-	uint32_t buffered, quantum, index;
+	uint32_t buffered, quantum, index, rate_size;
 	int32_t avail_buffers;
 
 	do {
 		seq1 = SPA_SEQ_READ(impl->seq);
 		memcpy(time, &impl->time, SPA_MIN(size, sizeof(struct pw_time)));
 		buffered = impl->rate_queued;
+		rate_size = impl->rate_size;
 		quantum = impl->quantum;
 		seq2 = SPA_SEQ_READ(impl->seq);
 	} while (!SPA_SEQ_READ_SUCCESS(seq1, seq2));
@@ -2374,8 +2388,10 @@ int pw_stream_get_time_n(struct pw_stream *stream, struct pw_time *time, size_t
 		time->buffered = buffered;
 	if (size >= offsetof(struct pw_time, avail_buffers))
 		time->queued_buffers = impl->n_buffers - avail_buffers;
-	if (size >= sizeof(struct pw_time))
+	if (size >= offsetof(struct pw_time, size))
 		time->avail_buffers = avail_buffers;
+	if (size >= sizeof(struct pw_time))
+		time->size = rate_size;
 
 	pw_log_trace_fp("%p: %"PRIi64" %"PRIi64" %"PRIu64" %d/%d %"PRIu64" %"
 			PRIu64" %"PRIu64" %"PRIu64" %"PRIu64" %d/%d", stream,
