@@ -931,11 +931,11 @@ static void do_print_delay(void *userdata, uint64_t expirations)
 	pw_stream_get_time_n(data->stream, &time, sizeof(time));
 	printf("stream time: now:%"PRIi64" rate:%u/%u ticks:%"PRIu64
 			" delay:%"PRIi64" queued:%"PRIu64
-			" buffered:%"PRIi64" buffers:%u avail:%u\n",
+			" buffered:%"PRIi64" buffers:%u avail:%u size:%"PRIu64"\n",
 		time.now,
 		time.rate.num, time.rate.denom,
 		time.ticks, time.delay, time.queued, time.buffered,
-		time.queued_buffers, time.avail_buffers);
+		time.queued_buffers, time.avail_buffers, time.size);
 }
 
 enum {
