@@ -1541,7 +1541,7 @@ int main(int argc, char *argv[])
 	pw_init(&argc, &argv);
 
 	data.out = stdout;
-	if (isatty(fileno(data.out)) && getenv("NO_COLOR") == NULL)
+	if (getenv("NO_COLOR") == NULL && isatty(fileno(data.out)))
 		colors = true;
 	setlinebuf(data.out);
 
@@ -1608,7 +1608,8 @@ int main(int argc, char *argv[])
 
 	data.core = pw_context_connect(data.context,
 			pw_properties_new(
-				PW_KEY_REMOTE_NAME, opt_remote,
+				PW_KEY_REMOTE_NAME, opt_remote ? opt_remote :
+					("[" PW_DEFAULT_REMOTE "-manager," PW_DEFAULT_REMOTE "]"),
 				NULL),
 			0);
 	if (data.core == NULL) {
