@@ -232,7 +232,7 @@ PWTEST(logger_debug_env_alpha)
 	enum spa_log_level level = pwtest_get_iteration(current_test);
 	enum spa_log_level default_level = pw_log_level;
 	struct spa_log *default_logger = pw_log_get();
-	char *lvl = NULL;
+	const char *lvl = NULL;
 	char *oldenv = getenv("PIPEWIRE_DEBUG");
 
 	if (oldenv)
@@ -276,7 +276,7 @@ PWTEST(logger_debug_env_topic_all)
 	struct spa_log *default_logger = pw_log_get();
 	char *oldenv = getenv("PIPEWIRE_DEBUG");
 	char lvlstr[32];
-	char *lvl = "X";
+	const char *lvl = "X";
 
 	if (oldenv)
 		oldenv = strdup(oldenv);
